@@ -3,7 +3,7 @@ vim: noexpandtab:ts=4:sts=4:sw=4
 
 nudoku
 
-Copyright (C) 2014 - 2018 Michael "jubalh" Vetter - jubalh _a-t_ iodoru.org
+Copyright (C) 2014 - 2020 Michael "jubalh" Vetter - jubalh _a-t_ iodoru.org
 
 LICENCE:
 This program is free software: you can redistribute it and/or modify
@@ -19,12 +19,18 @@ along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* INCLUDES */
+#include "gettext.h"			/* gettext */
 #include <stdlib.h>				/* rand, srand */
 #include <unistd.h>				/* getopt */
 #include <ncurses.h>			/* ncurses */
 #include <time.h>				/* time */
 #include <string.h>				/* strcmp, strlen */
 #include "sudoku.h"				/* sudoku functions */
+#ifdef ENABLE_CAIRO
+#include "outp.h"				/* output functions */
+#endif
+
+#define _(x) gettext(x)
 
 /* DEFINES */
 //#define VERSION				"0.1" //gets set via autotools
@@ -32,8 +38,8 @@ along with this program. If not, see <http://www.gnu.org/licenses/>.
 #define GRID_COLS				37
 #define GRID_Y					3
 #define GRID_X					3
-#define INFO_LINES				19
-#define INFO_COLS				20
+#define INFO_LINES				25
+#define INFO_COLS				30
 #define INFO_Y					3
 #define INFO_X					GRID_X + GRID_COLS + 5
 #define GRID_NUMBER_START_Y		1
@@ -54,13 +60,16 @@ along with this program. If not, see <http://www.gnu.org/licenses/>.
 #endif // DEBUG
 
 /* GLOBALS */
-static bool g_useColor = true;
-static bool g_playing = false;
-static bool g_useHighlights = false;
-static char* g_provided_stream;				/* in case of -s flag the user provides the sudoku stream */
-static int g_hint_counter;
-static char plain_board[STREAM_LENGTH];
-static char user_board[STREAM_LENGTH];
+static bool  g_useColor = true;
+static bool  g_playing = false;
+static bool  g_useHighlights = false;
+static char* g_provided_stream = NULL;		/* in case of -s flag the user provides the sudoku stream */
+static int   g_hint_counter;
+static char  plain_board[STREAM_LENGTH];
+static char  user_board[STREAM_LENGTH];
+static char* g_outputFilename = NULL;		/* in case -p/-i flag we get a filename passed for outputting */
+static int   g_sudokuCount = 1;				/* in case of -n we can the numbers of sudoku that should end up in the PDf (-p) */
+static bool  g_outIsPDF;
 static DIFFICULTY g_level = D_EASY;
 static WINDOW *grid, *infobox, *status;
 
@@ -68,7 +77,7 @@ static WINDOW *grid, *infobox, *status;
 static void print_version(void)
 {
 	printf("nudoku version " VERSION "\n\n\
-Copyright (C) Michael Vetter 2014\n\
+Copyright (C) Michael Vetter 2014 - 2020\n\
 License GPLv3+: GNU GPL version 3 or later.\n\
 This is free software, you are free to modify and redistribute it.\n");
 #ifdef DEBUG
@@ -78,13 +87,16 @@ This is free software, you are free to modify and redistribute it.\n");
 
 static void print_usage(void)
 {
-	printf("nudoku [option]\n\n");
-	printf("Options:\n");
-	printf("-h help:\t\tPrint this help\n");
-	printf("-v version:\t\tPrint version\n");
-	printf("-c nocolor:\t\tDo not use colors\n");
-	printf("-d difficulty:\t\tChoose between: easy, normal, hard\n");
-	printf("-s stream:\t\tUser provided sudoku stream\n");
+	printf(_("nudoku [option]\n\n"));
+	printf(_("Options:\n"));
+	printf(_("-h help:\t\tPrint this help\n"));
+	printf(_("-v version:\t\tPrint version\n"));
+	printf(_("-c nocolor:\t\tDo not use colors\n"));
+	printf(_("-d difficulty:\t\tChoose between: easy, normal, hard\n"));
+	printf(_("-s stream:\t\tUser provided sudoku stream\n"));
+	printf(_("-p filename:\t\tOutput PDF\n"));
+	printf(_("-n filename:\t\tNumber of sudokus to put in PDF\n"));
+	printf(_("-i filename:\t\tOutput PNG image\n"));
 }
 
 static bool is_valid_stream(char *s)
@@ -98,7 +110,7 @@ static bool is_valid_stream(char *s)
 
 		if(!((*p >= 49 && *p <= 57) || *p == '.' ))
 		{
-			printf("Character %c at position %d is not allowed.\n", *p, n);
+			printf(_("Character %c at position %d is not allowed.\n"), *p, n);
 			return false;
 		}
 		p++;
@@ -106,13 +118,13 @@ static bool is_valid_stream(char *s)
 
 	if (n != SUDOKU_LENGTH )
 	{
-		printf("Stream has to be %d characters long.\n", SUDOKU_LENGTH);
+		printf(_("Stream has to be %d characters long.\n"), SUDOKU_LENGTH);
 		return false;
 	}
 
 	if (!is_valid_puzzle(s))
 	{
-		printf("Stream does not represent a valid sudoku puzzle.\n");
+		printf(_("Stream does not represent a valid sudoku puzzle.\n"));
 		return false;
 	}
 
@@ -122,7 +134,7 @@ static bool is_valid_stream(char *s)
 static void parse_arguments(int argc, char *argv[])
 {
 	int opt;
-	while ((opt = getopt(argc, argv, "hvcs:d:")) != -1)
+	while ((opt = getopt(argc, argv, "hvcs:d:p:i:n:")) != -1)
 	{
 		switch (opt)
 		{
@@ -153,6 +165,20 @@ static void parse_arguments(int argc, char *argv[])
 					exit(EXIT_FAILURE);
 				}
 				break;
+			// output pdf
+			case 'p':
+				g_outputFilename = strdup(optarg);
+				g_outIsPDF = true;
+				break;
+			// output png image
+			case 'i':
+				g_outputFilename = strdup(optarg);
+				g_outIsPDF = false;
+				break;
+			// numbers of sudoku for output pdf
+			case 'n':
+				g_sudokuCount = atoi(optarg);
+				break;
 			default:
 				print_usage();
 				exit(EXIT_FAILURE);
@@ -168,6 +194,7 @@ static void cleanup(void)
 static void init_curses(void)
 {
 	initscr();
+	use_default_colors();
 	clear();
 	atexit(cleanup);
 	cbreak();
@@ -178,10 +205,10 @@ static void init_curses(void)
 		if(has_colors())
 		{
 			start_color();
-			init_pair(1, COLOR_GREEN, COLOR_BLACK);
-			init_pair(2, COLOR_BLUE, COLOR_BLACK);
+			init_pair(1, COLOR_GREEN, -1);
+			init_pair(2, COLOR_BLUE, -1);
 			// user input color
-			init_pair(3, COLOR_CYAN, COLOR_BLACK);
+			init_pair(3, COLOR_CYAN, -1);
 			// Highlight color
 			init_pair(COLOR_HIGHLIGHT, COLOR_BLACK, COLOR_WHITE);
 			// Cursor highlight color
@@ -189,7 +216,7 @@ static void init_curses(void)
 		}
 		else
 		{
-			printw( "Your terminal doesn't support colors.\nTry the nocolor (-c) option.\n");
+			printw( _("Your terminal doesn't support colors.\nTry the nocolor (-c) option.\n"));
 			getch();
 			exit(EXIT_FAILURE);
 		}
@@ -211,13 +238,36 @@ static void _draw_grid()
 				if(j % 3 == 0)
 					wattron(grid, A_BOLD|COLOR_PAIR(2));
 			}
-			wprintw(grid, "+");
+
+			if(i == 0 && j == 0)
+				waddch(grid, ACS_ULCORNER);
+			else if(i == 0 && j == 9)
+				waddch(grid, ACS_URCORNER);
+			else if(i == 9 && j == 9)
+				waddch(grid, ACS_LRCORNER);
+			else if(i == 9 && j == 0)
+				waddch(grid, ACS_LLCORNER);
+			else if(i == 0)
+				waddch(grid, ACS_TTEE);
+			else if(i == 9)
+				waddch(grid, ACS_BTEE);
+			else if(j == 0)
+				waddch(grid, ACS_LTEE);
+			else if(j == 9)
+				waddch(grid, ACS_RTEE);
+			else
+				waddch(grid, ACS_PLUS);
+
 			if(g_useColor && (j % 3 == 0) && (i % 3 != 0))
 			{
 				wattron(grid, A_BOLD|COLOR_PAIR(1));
 			}
 			if(j < 9)
-				wprintw(grid, "---");
+			{
+				waddch(grid, ACS_HLINE);
+				waddch(grid, ACS_HLINE);
+				waddch(grid, ACS_HLINE);
+			}
 			if(g_useColor && (i % 3 == 0))
 			{
 				wattron(grid, A_BOLD|COLOR_PAIR(1));
@@ -228,9 +278,8 @@ static void _draw_grid()
 			if(g_useColor && (j % 3 == 0))
 				wattron(grid, A_BOLD|COLOR_PAIR(2));
 			if(j > 0)
-				wprintw(grid, "   |");
-			else
-				wprintw(grid, "|");
+				wprintw(grid, "   ");
+			waddch(grid, ACS_VLINE);
 			if(g_useColor && (j % 3 == 0))
 			{
 				wattron(grid, A_BOLD|COLOR_PAIR(1));
@@ -252,15 +301,13 @@ static void init_windows(void)
 	if (g_useColor)
 	{
 		wbkgd(infobox, COLOR_PAIR(2));
-	}
-
-	if (g_useColor)
 		wattron(infobox, A_BOLD|COLOR_PAIR(2));
+	}
 
 	wprintw(infobox, "nudoku %s\n", VERSION);
 
 	if (!g_provided_stream)
-		wprintw(infobox, "level: %s\n\n", difficulty_to_str(g_level) );
+		wprintw(infobox, _("level: %s\n\n"), difficulty_to_str(g_level) );
 	else
 		wprintw(infobox, "\n\n");
 
@@ -269,23 +316,23 @@ static void init_windows(void)
 		wattroff(infobox, A_BOLD|COLOR_PAIR(2));
 		wattron(infobox, COLOR_PAIR(1));
 	}
-	wprintw(infobox, "Movement\n");
-	wprintw(infobox, " h - Move left\n");
-	wprintw(infobox, " j - Move down\n");
-	wprintw(infobox, " k - Move up\n");
-	wprintw(infobox, " l - Move right\n\n");
-	wprintw(infobox, "Commands\n");
-	wprintw(infobox, " c - Check solution\n");
-	wprintw(infobox, " H - Give a hint\n");
+	wprintw(infobox, _("Movement\n"));
+	wprintw(infobox, _(" h - Move left\n"));
+	wprintw(infobox, _(" j - Move down\n"));
+	wprintw(infobox, _(" k - Move up\n"));
+	wprintw(infobox, _(" l - Move right\n\n"));
+	wprintw(infobox, _("Commands\n"));
+	wprintw(infobox, _(" c - Check solution\n"));
+	wprintw(infobox, _(" H - Give a hint\n"));
 	if (g_useColor)
 	{
-		wprintw(infobox, " m - Toggle marks\n");
+		wprintw(infobox, _(" m - Toggle marks\n"));
 	}
-	wprintw(infobox, " N - New puzzle\n");
-	wprintw(infobox, " Q - Quit\n");
-	wprintw(infobox, " r - Redraw\n");
-	wprintw(infobox, " S - Solve puzzle\n");
-	wprintw(infobox, " x - Delete number\n");
+	wprintw(infobox, _(" N - New puzzle\n"));
+	wprintw(infobox, _(" Q - Quit\n"));
+	wprintw(infobox, _(" r - Redraw\n"));
+	wprintw(infobox, _(" S - Solve puzzle\n"));
+	wprintw(infobox, _(" x - Delete number\n"));
 	if (g_useColor)
 	{
 		wattroff(infobox, COLOR_PAIR(1));
@@ -300,14 +347,16 @@ static int get_character_at_grid(char* board, int x, int y)
 	return board[posy*9+posx];
 }
 
-static void fill_grid(char *board, int x_cursor, int y_cursor)
+static void fill_grid(char *user_board, char *plain_board, int x_cursor, int y_cursor)
 {
 	int row, col, x, y;
 	int n;
 	int c;
 	int selected;
+	bool isUserInput;
+	int m;
 
-	selected = get_character_at_grid(board, x_cursor, y_cursor);
+	selected = get_character_at_grid(user_board, x_cursor, y_cursor);
 	wstandend(grid);
 	y = GRID_NUMBER_START_Y;
 	for(row=0; row < 9; row++)
@@ -315,11 +364,17 @@ static void fill_grid(char *board, int x_cursor, int y_cursor)
 		x = GRID_NUMBER_START_X;
 		for(col=0; col < 9; col++)
 		{
-			n = board[row*9+col];
+			isUserInput = true;
+			n = user_board[row*9+col];
 			if(n == '.')
 				c = ' ';
 			else
+			{
 				c = n;
+				m = plain_board[row*9+col];
+				if(n == m)
+					isUserInput = false;
+			}
 			if (g_useColor && g_useHighlights && selected == c)
 			{
 				if (x == x_cursor && y == y_cursor)
@@ -327,7 +382,11 @@ static void fill_grid(char *board, int x_cursor, int y_cursor)
 				else
 					wattron(grid, COLOR_PAIR(COLOR_HIGHLIGHT));
 			}
+			if(isUserInput)
+				wattron(grid, COLOR_PAIR(3));
 			mvwprintw(grid, y, x, "%c", c);
+			if(isUserInput)
+				wattroff(grid, COLOR_PAIR(3));
 			if (g_useColor && g_useHighlights && selected == c)
 			{
 				if (x == x_cursor && y == y_cursor)
@@ -343,7 +402,7 @@ static void fill_grid(char *board, int x_cursor, int y_cursor)
 
 static void new_puzzle(void)
 {
-	int holes = get_holes(g_level);
+	int holes = g_level;
 	char* stream;
 
 	if (g_provided_stream)
@@ -358,7 +417,7 @@ static void new_puzzle(void)
 	if (!g_provided_stream)
 		free(stream);
 
-	fill_grid(plain_board, GRID_NUMBER_START_X, GRID_NUMBER_START_Y);
+	fill_grid(plain_board, plain_board, GRID_NUMBER_START_X, GRID_NUMBER_START_Y);
 
 	g_playing = true;
 }
@@ -389,20 +448,37 @@ static bool hint(void)
 
 int main(int argc, char *argv[])
 {
-	bool run = true, enable_highlights=false;
+#if ENABLE_NLS
+	/* Set up internationalization */
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+#endif
+	bool run = true;
 	int key, x, y, posx, posy;
 
-	g_provided_stream = NULL;
-
 	parse_arguments(argc, argv);
+	srand(time(NULL));
+
+	if (g_outputFilename)
+	{
+#ifdef ENABLE_CAIRO
+		generate_output(g_level, g_outputFilename, g_sudokuCount, g_outIsPDF);
+		return 0;
+#else
+		printf("nudoku is compiled without cairo support.\n");
+		printf("To use the output feature, please compile with --enable-cairo.\n");
+		return 1;
+#endif
+	}
+
 	init_curses();
 	init_windows();
-	srand(time(NULL));
 
 #ifdef DEBUG
 	strcpy(plain_board, EXAMPLE_STREAM);
 	strcpy(user_board, EXAMPLE_STREAM);
-	fill_grid(plain_board, GRID_NUMBER_START_X, GRID_NUMBER_START_Y);
+	fill_grid(plain_board, plain_board, GRID_NUMBER_START_X, GRID_NUMBER_START_Y);
 	g_playing = true;
 #else
 	new_puzzle();
@@ -439,7 +515,7 @@ int main(int argc, char *argv[])
 						// this should only be done when we are playing, because plain_board
 						// is actually the one being solved and thus displayed.
 						// this is true for all movement keys.
-						fill_grid(user_board, x, y);
+						fill_grid(user_board, plain_board, x, y);
 					}
 				}
 				break;
@@ -450,7 +526,7 @@ int main(int argc, char *argv[])
 					x += GRID_LINE_DELTA;
 					if(g_playing)
 					{
-						fill_grid(user_board, x, y);
+						fill_grid(user_board, plain_board, x, y);
 					}
 				}
 				break;
@@ -461,7 +537,7 @@ int main(int argc, char *argv[])
 					y -= GRID_COL_DELTA;
 					if(g_playing)
 					{
-						fill_grid(user_board, x, y);
+						fill_grid(user_board, plain_board, x, y);
 					}
 				}
 				break;
@@ -472,7 +548,7 @@ int main(int argc, char *argv[])
 					y += GRID_COL_DELTA;
 					if(g_playing)
 					{
-						fill_grid(user_board, x, y);
+						fill_grid(user_board, plain_board, x, y);
 					}
 				}
 				break;
@@ -490,13 +566,13 @@ int main(int argc, char *argv[])
 				{
 					g_useHighlights = false;
 					werase(status);
-					mvwprintw(status, 0, 0, "Solving puzzle...");
+					mvwprintw(status, 0, 0, _("Solving puzzle..."));
 					refresh();
 					wrefresh(status);
 					solve(plain_board);
-					fill_grid(plain_board, x, y);
+					fill_grid(plain_board, plain_board, x, y);
 					werase(status);
-					mvwprintw(status, 0, 0, "Solved");
+					mvwprintw(status, 0, 0, _("Solved"));
 					g_playing = false;
 				}
 				break;
@@ -505,7 +581,7 @@ int main(int argc, char *argv[])
 				g_hint_counter = 0;
 
 				werase(status);
-				mvwprintw(status, 0, 0, "Generating puzzle...");
+				mvwprintw(status, 0, 0, _("Generating puzzle..."));
 				refresh();
 				wrefresh(status);
 				new_puzzle();
@@ -531,18 +607,18 @@ int main(int argc, char *argv[])
 
 					if(solvable == 0)
 					{
-						mvwprintw(status, 0, 0, "Not correct");
+						mvwprintw(status, 0, 0, _("Not correct"));
 					}
 					else
 					{
 						if (strchr(user_board, '.') == NULL)
 						{
-							mvwprintw(status, 0, 0, "Solved");
+							mvwprintw(status, 0, 0, _("Solved"));
 
 							if (g_hint_counter > 0)
 							{
 								char t[256];
-								sprintf(t, " with the help of %d hints", g_hint_counter);
+								sprintf(t, _(" with the help of %d hints"), g_hint_counter);
 								mvwprintw(status, 0, 6, t);
 							}
 
@@ -550,7 +626,7 @@ int main(int argc, char *argv[])
 						}
 						else
 						{
-							mvwprintw(status, 0, 0, "Correct so far");
+							mvwprintw(status, 0, 0, _("Correct so far"));
 						}
 					}
 				}
@@ -576,9 +652,9 @@ int main(int argc, char *argv[])
 				if (g_playing && hint())
 				{
 					g_hint_counter++;
-					fill_grid(user_board, x, y);
+					fill_grid(user_board, plain_board, x, y);
 					werase(status);
-					mvwprintw(status, 0, 0, "Provided hint");
+					mvwprintw(status, 0, 0, _("Provided hint"));
 				}
 				break;
 			case 'm':
@@ -586,7 +662,7 @@ int main(int argc, char *argv[])
 				if (g_useColor)
 				{
 					g_useHighlights = !g_useHighlights;
-					fill_grid(user_board, x, y);
+					fill_grid(user_board, plain_board, x, y);
 				}
 				break;
 			default:
@@ -603,7 +679,7 @@ int main(int argc, char *argv[])
 				// add inputted number to grid
 				user_board[posy*9+posx] = key;
 				// redraw grid to update highlight
-				fill_grid(user_board, x, y);
+				fill_grid(user_board, plain_board, x, y);
 			}
 		}
 		wmove(grid, y,x);
@@ -615,6 +691,8 @@ int main(int argc, char *argv[])
 
 	if (g_provided_stream)
 		free(g_provided_stream);
+	if (g_outputFilename)
+		free(g_outputFilename);
 
 	endwin();
 	return EXIT_SUCCESS;
