@@ -3,7 +3,7 @@ vim: noexpandtab:ts=4:sts=4:sw=4
 
 nudoku
 
-Copyright (C) 2014 - 2018 Michael "jubalh" Vetter - jubalh _a-t_ iodoru.org
+Copyright (C) 2014 - 2020 Michael "jubalh" Vetter - jubalh _a-t_ iodoru.org
 
 LICENCE:
 This program is free software: you can redistribute it and/or modify
@@ -23,6 +23,10 @@ along with this program. If not, see <http://www.gnu.org/licenses/>.
 #include <time.h>		/* time */
 #include <string.h>		/* strdup */
 #include "sudoku.h"		/* enum */
+#include "gettext.h"	/* gettext */
+#include "unistd.h"		/* sleep */
+
+#define _(x) gettext(x)
 
 /* FUNCTIONS */
 
@@ -80,13 +84,15 @@ static bool is_available(char puzzle[STREAM_LENGTH], int row, int col, int num)
 	int rowStart = (row/3) * 3;
 	int colStart = (col/3) * 3;
 
+	num += 48;
+
 	for (i=0; i<9; i++)
 	{
-		if (puzzle[row * 9 + i] - 48 == num)
+		if (puzzle[row * 9 + i] == num)
 			return false;
-		if (puzzle[i * 9 + col] - 48 == num)
+		if (puzzle[i * 9 + col] == num)
 			return false;
-		if (puzzle[(rowStart + (i % 3)) * 9 + (colStart + (i / 3))] - 48 == num)
+		if (puzzle[(rowStart + (i % 3)) * 9 + (colStart + (i / 3))] == num)
 			return false;
 	}
 	return true;
@@ -172,8 +178,6 @@ static char* generate_seed()
 	int index = 0;
 	int iSquare = 0;
 
-	srand(time(NULL));
-
 	char* upperleft = create_random_numbers();
 	char* center = create_random_numbers();
 	char* lowerright = create_random_numbers();
@@ -204,7 +208,7 @@ static char* generate_seed()
 		for(int j = 0; j < 6 ; j++)
 			stream[index++] = '.';
 		for(int j = 0; j < 3; j++)
-			stream[index++] = upperleft[iSquare++];
+			stream[index++] = lowerright[iSquare++];
 	}
 
 	stream[81] = '\0';
@@ -230,40 +234,20 @@ static void punch_holes(char *stream, int count)
 	}
 }
 
-char* difficulty_to_str(DIFFICULTY level)
+const char* difficulty_to_str(DIFFICULTY level)
 {
 	switch(level)
 	{
 		case D_HARD:
-			return "hard";
+			return _("hard");
 		case D_NORMAL:
-			return "normal";
+			return _("normal");
 		case D_EASY:
 		default:
-			return "easy";
+			return _("easy");
 	}
 }
 
-int get_holes(DIFFICULTY level)
-{
-	int holes;
-	switch (level)
-	{
-		case D_HARD:
-			/*holes = rand() % 60 + 30;*/
-			holes = 50;
-			break;
-		case D_NORMAL:
-			holes = 40;
-			break;
-		case D_EASY:
-		default:
-			holes = 30;
-			break;
-	}
-	return holes;
-}
-
 char* generate_puzzle(int holes)
 {
 	char* stream;
